@@ -38,8 +38,6 @@ ZLIB_VERSION=${ZLIB_VERSION:-"1.2.8"}
 SQLITE_VERSION=${SQLITE_VERSION:-"3.8.11.1"}
 # Used to construct the SQLite download URL.
 SQLITE_VERSION_REL_YEAR=2015
-GTEST_VERSION=${GMOCK_VERSION:-"1.7.0"}
-GMOCK_VERSION=${GMOCK_VERSION:-"1.7.0"}
 HTTPD_VERSION=${HTTPD_VERSION:-"2.4.10"}
 APR_ICONV_VERSION=${APR_ICONV_VERSION:-"1.2.1"}
 
@@ -49,10 +47,6 @@ SERF=serf-${SERF_VERSION}
 ZLIB=zlib-${ZLIB_VERSION}
 SQLITE_VERSION_LIST=`echo $SQLITE_VERSION | sed -e 's/\./ /g'`
 SQLITE=sqlite-amalgamation-`printf %d%02d%02d%02d $SQLITE_VERSION_LIST`
-GTEST=release-${GTEST_VERSION}
-GTEST_URL=https://github.com/google/googletest/archive
-GMOCK=release-${GMOCK_VERSION}
-GMOCK_URL=https://github.com/google/googlemock/archive
 
 HTTPD=httpd-${HTTPD_VERSION}
 APR_ICONV=apr-iconv-${APR_ICONV_VERSION}
@@ -67,12 +61,12 @@ HTTP_FETCH=
 
 # Need this uncommented if any of the specific versions of the ASF tarballs to
 # be downloaded are no longer available on the general mirrors.
-APACHE_MIRROR=http://archive.apache.org/dist
+APACHE_MIRROR=https://archive.apache.org/dist
 
 # helpers
 usage() {
     echo "Usage: $0"
-    echo "Usage: $0 [ apr | serf | zlib | sqlite | googlemock ] ..."
+    echo "Usage: $0 [ apr | serf | zlib | sqlite ] ..."
     exit $1
 }
 
@@ -106,7 +100,7 @@ get_zlib() {
     test -d $BASEDIR/zlib && return
 
     cd $TEMPDIR
-    $HTTP_FETCH http://sourceforge.net/projects/libpng/files/zlib/$ZLIB_VERSION/$ZLIB.tar.gz
+    $HTTP_FETCH https://sourceforge.net/projects/libpng/files/zlib/$ZLIB_VERSION/$ZLIB.tar.gz
     cd $BASEDIR
 
     gzip -dc $TEMPDIR/$ZLIB.tar.gz | tar -xf -
@@ -127,29 +121,11 @@ get_sqlite() {
 
 }
 
-get_googlemock() {
-    test -d $BASEDIR/googlemock && return
-
-    cd $TEMPDIR
-    $HTTP_FETCH ${GTEST_URL}/${GTEST}.zip
-    unzip -q ${GTEST}.zip
-    rm -f ${GTEST}.zip
-
-    $HTTP_FETCH ${GMOCK_URL}/${GMOCK}.zip
-    unzip -q ${GMOCK}.zip
-    rm -f ${GMOCK}.zip
-
-    cd $BASEDIR
-    mkdir googlemock
-    mv $TEMPDIR/googletest-release-${GTEST_VERSION} googlemock/googletest
-    mv $TEMPDIR/googlemock-release-${GMOCK_VERSION} googlemock/googlemock
-}
-
 # main()
 get_deps() {
     mkdir -p $TEMPDIR
 
-    for i in zlib serf sqlite-amalgamation apr apr-util gmock-fused; do
+    for i in zlib serf sqlite-amalgamation apr apr-util; do
       if [ -d $i ]; then
         echo "Local directory '$i' already exists; the downloaded copy won't be used" >&2
       fi
